@@ -625,8 +625,10 @@ SANITY();
 - (void) setAttributes: (NSDictionary*)attributes
 		 range: (NSRange)range
 {
-  unsigned	tmpLength, arrayIndex, arraySize;
-  NSRange	effectiveRange;
+  unsigned	tmpLength;
+  unsigned	arrayIndex = 0;
+  unsigned	arraySize;
+  NSRange	effectiveRange = NSMakeRange(0, NSNotFound);
   unsigned	afterRangeLoc, beginRangeLoc;
   NSDictionary	*attrs;
   NSZone	*z = [self zone];
@@ -751,8 +753,10 @@ SANITY();
 - (void) replaceCharactersInRange: (NSRange)range
 		       withString: (NSString*)aString
 {
-  unsigned	tmpLength, arrayIndex, arraySize;
-  NSRange	effectiveRange;
+  unsigned	tmpLength;
+  unsigned	arrayIndex = 0;
+  unsigned	arraySize;
+  NSRange	effectiveRange = NSMakeRange(0, NSNotFound);
   NSDictionary	*attrs;
   GSAttrInfo	*info;
   int		moveLocations;
