@@ -22,20 +22,18 @@
    Boston, MA 02111 USA.
    */
 
-#import "config.h"
+#import "common.h"
 #import "Foundation/NSSet.h"
 #import "Foundation/NSAutoreleasePool.h"
 #import "Foundation/NSEnumerator.h"
 #import "Foundation/NSException.h"
-#import "Foundation/NSString.h"
 #import "Foundation/NSPortCoder.h"
-#import "Foundation/NSDebug.h"
 
 
 #define	GSI_MAP_RETAIN_VAL(M, X)	
 #define	GSI_MAP_RELEASE_VAL(M, X)	
 #define GSI_MAP_KTYPES	GSUNION_OBJ
-#define GSI_MAP_VTYPES	GSUNION_INT
+#define GSI_MAP_VTYPES	GSUNION_NSINT
 
 #if	GS_WITH_GC
 #include	<gc_typed.h>
@@ -50,6 +48,8 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 {
 @public
   GSIMapTable_t	map;
+@private
+  NSUInteger _version;
 }
 @end
 
@@ -127,23 +127,25 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 		  format: @"Tried to nil value to counted set"];
     }
 
+  _version++;
   node = GSIMapNodeForKey(&map, (GSIMapKey)anObject);
   if (node == 0)
     {
-      GSIMapAddPair(&map,(GSIMapKey)anObject,(GSIMapVal)(unsigned)1);
+      GSIMapAddPair(&map,(GSIMapKey)anObject,(GSIMapVal)(NSUInteger)1);
     }
   else
     {
-      node->value.uint++;
+      node->value.nsu++;
     }
+  _version++;
 }
 
-- (unsigned) count
+- (NSUInteger) count
 {
   return map.nodeCount;
 }
 
-- (unsigned) countForObject: (id)anObject
+- (NSUInteger) countForObject: (id)anObject
 {
   if (anObject)
     {
@@ -151,7 +153,7 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 
       if (node)
 	{
-	  return node->value.uint;
+	  return node->value.nsu;
 	}
     }
   return 0;
@@ -179,13 +181,13 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
   while (node != 0)
     {
       (*imp1)(aCoder, sel1, node->key.obj);
-      (*imp2)(aCoder, sel2, type, &node->value.uint);
+      (*imp2)(aCoder, sel2, type, &node->value.nsu);
       node = GSIMapEnumeratorNextNode(&enumerator);
     }
   GSIMapEndEnumerator(&enumerator);
 }
 
-- (unsigned) hash
+- (NSUInteger) hash
 {
   return map.nodeCount;
 }
@@ -196,7 +198,7 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 }
 
 /* Designated initialiser */
-- (id) initWithCapacity: (unsigned)cap
+- (id) initWithCapacity: (NSUInteger)cap
 {
   GSIMapInitWithZoneAndCapacity(&map, [self zone], cap);
   return self;
@@ -206,7 +208,7 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 {
   unsigned	count;
   id		value;
-  unsigned	valcnt;
+  NSUInteger	valcnt;
   SEL		sel = @selector(decodeValueOfObjCType:at:);
   IMP		imp = [aCoder methodForSelector: sel];
   const char	*utype = @encode(unsigned);
@@ -225,9 +227,9 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
   return self;
 }
 
-- (id) initWithObjects: (id*)objs count: (unsigned)c
+- (id) initWithObjects: (id*)objs count: (NSUInteger)c
 {
-  unsigned int	i;
+  NSUInteger	i;
 
   self = [self initWithCapacity: c];
   if (self == nil)
@@ -240,18 +242,18 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 
       if (objs[i] == nil)
 	{
-	  IF_NO_GC(AUTORELEASE(self));
+	  DESTROY(self);
 	  [NSException raise: NSInvalidArgumentException
 		      format: @"Tried to init counted set with nil value"];
 	}
       node = GSIMapNodeForKey(&map, (GSIMapKey)objs[i]);
       if (node == 0)
 	{
-	  GSIMapAddPair(&map,(GSIMapKey)objs[i],(GSIMapVal)(unsigned)1);
+	  GSIMapAddPair(&map,(GSIMapKey)objs[i],(GSIMapVal)(NSUInteger)1);
         }
       else
 	{
-	  node->value.uint++;
+	  node->value.nsu++;
 	}
     }
   return self;
@@ -284,7 +286,7 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
  * of the GSIMap enumeration that, once enumerated, an object can be removed
  * from the map.  If GSIMap ever loses that characterstic, this will break.
  */
-- (void) purge: (int)level
+- (void) purge: (NSInteger)level
 {
   if (level > 0)
     {
@@ -294,10 +296,12 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 
       while (node != 0)
 	{
-	  if (node->value.uint <= (unsigned int)level)
+	  if (node->value.nsu <= (NSUInteger)level)
 	    {
+	      _version++;
 	      GSIMapRemoveNodeFromMap(&map, bucket, node);
 	      GSIMapFreeNode(&map, node);
+	      _version++;
 	    }
 	  bucket = GSIMapEnumeratorBucket(&enumerator);
 	  node = GSIMapEnumeratorNextNode(&enumerator);
@@ -308,7 +312,9 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 
 - (void) removeAllObjects
 {
+  _version++;
   GSIMapCleanMap(&map);
+  _version++;
 }
 
 /**
@@ -326,6 +332,7 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
       NSWarnMLog(@"attempt to remove nil object");
       return;
     }
+  _version++;
   bucket = GSIMapBucketForKey(&map, (GSIMapKey)anObject);
   if (bucket != 0)
     {
@@ -334,19 +341,21 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
       node = GSIMapNodeForKeyInBucket(&map, bucket, (GSIMapKey)anObject);
       if (node != 0)
 	{
-	  if (--node->value.uint == 0)
+	  if (--node->value.nsu == 0)
 	    {
 	      GSIMapRemoveNodeFromMap(&map, bucket, node);
 	      GSIMapFreeNode(&map, node);
 	    }
 	}
     }
+  _version++;
 }
 
 - (id) unique: (id)anObject
 {
   GSIMapNode	node;
   id		result;
+  _version++;
 
   if (anObject == nil)
     {
@@ -358,12 +367,12 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
   if (node == 0)
     {
       result = anObject;
-      GSIMapAddPair(&map,(GSIMapKey)anObject,(GSIMapVal)(unsigned)1);
+      GSIMapAddPair(&map,(GSIMapKey)anObject,(GSIMapVal)(NSUInteger)1);
     }
   else
     {
       result = node->key.obj;
-      node->value.uint++;
+      node->value.nsu++;
 #if	!GS_WITH_GC
       if (result != anObject)
 	{
@@ -372,6 +381,16 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 	}
 #endif
     }
+  _version++;
   return result;
 }
+
+- (NSUInteger) countByEnumeratingWithState: (NSFastEnumerationState*)state
+                                   objects: (id*)stackbuf
+                                     count: (NSUInteger)len
+{
+  state->mutationsPtr = (unsigned long *)&_version;
+  return GSIMapCountByEnumeratingWithStateObjectsCount
+    (&map, state, stackbuf, len);
+}
 @end
