@@ -23,13 +23,12 @@
    Boston, MA 02111 USA.
 */
 
-#import "config.h"
+#import "common.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSDictionary.h"
 #import "Foundation/NSEnumerator.h"
 #import "Foundation/NSByteOrder.h"
 #import "Foundation/NSData.h"
-#import "Foundation/NSDebug.h"
 #import "Foundation/NSException.h"
 #import "Foundation/NSFileHandle.h"
 #import "Foundation/NSHost.h"
@@ -39,12 +38,13 @@
 #import "Foundation/NSPathUtilities.h"
 #import "Foundation/NSProcessInfo.h"
 #import "Foundation/NSRunLoop.h"
-#import "Foundation/NSString.h"
 #import "Foundation/NSURL.h"
 #import "Foundation/NSURLHandle.h"
 #import "Foundation/NSValue.h"
 #import "GNUstepBase/GSMime.h"
 #import "GNUstepBase/GSLock.h"
+#import "GNUstepBase/NSString+GNUstepBase.h"
+#import "GNUstepBase/NSURL+GNUstepBase.h"
 #import "NSCallBacks.h"
 #import "GSURLPrivate.h"
 #import "GSPrivate.h"
@@ -69,7 +69,7 @@
  * in case the remote server is buggy and requires particular
  * captialisation of headers (some http software is faulty like that).
  */
-static unsigned int
+static NSUInteger
 _id_hash(void *table, NSString* o)
 {
   return [[o uppercaseString] hash];
@@ -81,7 +81,7 @@ _id_is_equal(void *table, NSString *o, NSString *p)
   return ([o caseInsensitiveCompare: p] == NSOrderedSame) ? YES : NO;
 }
 
-typedef unsigned int (*NSMT_hash_func_t)(NSMapTable *, const void *);
+typedef NSUInteger (*NSMT_hash_func_t)(NSMapTable *, const void *);
 typedef BOOL (*NSMT_is_equal_func_t)(NSMapTable *, const void *, const void *);
 typedef void (*NSMT_retain_func_t)(NSMapTable *, const void *);
 typedef void (*NSMT_release_func_t)(NSMapTable *, void *);
@@ -313,7 +313,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 			     [[NSProcessInfo processInfo] processIdentifier]];
       IF_NO_GC([debugFile retain];)
 
-#if	!defined(__MINGW32__)
+#if	!defined(__MINGW__)
       sslClass = [NSFileHandle sslClass];
 #endif
     }
@@ -555,7 +555,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   [sock writeInBackgroundAndNotify: buf];
   RELEASE(buf);
   RELEASE(s);
-  RELEASE(self);
+  DESTROY(self);
 }
 
 - (void) bgdRead: (NSNotification*) not
@@ -585,7 +585,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	{
 	  NSLog(@"%@ %p %s Unexpected data (%*.*s) from remote!",
 	    NSStringFromSelector(_cmd), self, keepalive?"K":"",
-	    [d length], [d length], [d bytes]);
+	    (int)[d length], (int)[d length], [d bytes]);
 	}
       [nc removeObserver: self name: nil object: sock];
       [sock closeFile];
@@ -810,7 +810,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	    }
 	}
     }
-  RELEASE(self);
+  DESTROY(self);
 }
 
 - (void) bgdTunnelRead: (NSNotification*) not
@@ -862,7 +862,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	}
     }
   RELEASE(p);
-  RELEASE(self);
+  DESTROY(self);
 }
 
 - (void) loadInBackground
@@ -900,7 +900,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
   if (debug)
     NSLog(@"%@ %p %s", NSStringFromSelector(_cmd), self, keepalive?"K":"");
 
-  path = [[u fullPath] stringByTrimmingSpaces];
+  path = [[[u fullPath] stringByTrimmingSpaces]
+    stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
   if ([path length] == 0)
     {
       path = @"/";
@@ -920,7 +921,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       [self endLoadInBackground];
       [self backgroundLoadDidFailWithReason:
 	[NSString stringWithFormat: @"Failed to connect: %@", e]];
-      RELEASE(self);
+      DESTROY(self);
       return;
     }
 
@@ -1002,7 +1003,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	{
 	  [self endLoadInBackground];
 	  [self backgroundLoadDidFailWithReason: @"Failed proxy tunneling"];
-	  RELEASE(self);
+	  DESTROY(self);
 	  return;
 	}
     }
@@ -1018,7 +1019,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	  [self endLoadInBackground];
 	  [self backgroundLoadDidFailWithReason:
 	    @"Failed to make ssl connect"];
-	  RELEASE(self);
+	  DESTROY(self);
 	  return;
 	}
     }
@@ -1060,7 +1061,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 
   [self bgdApply: s];
   RELEASE(s);
-  RELEASE(self);
+  DESTROY(self);
 }
 
 - (void) bgdWrite: (NSNotification*)notification
@@ -1102,7 +1103,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
       [self endLoadInBackground];
       [self backgroundLoadDidFailWithReason:
 	[NSString stringWithFormat: @"Failed to write request: %@", e]];
-      RELEASE(self);
+      DESTROY(self);
       return;
     }
   else
@@ -1139,7 +1140,7 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	}
       connectionState = reading;
     }
-  RELEASE(self);
+  DESTROY(self);
 }
 
 /**
@@ -1487,7 +1488,8 @@ static void debugWrite(GSHTTPURLHandle *handle, NSData *data)
 	      method = @"GET";
 	    }
 	}
-      path = [[u fullPath] stringByTrimmingSpaces];
+      path = [[[u fullPath] stringByTrimmingSpaces]
+        stringByAddingPercentEscapesUsingEncoding: NSUTF8StringEncoding];
       if ([path length] == 0)
 	{
 	  path = @"/";
