@@ -22,20 +22,19 @@
    Boston, MA 02111 USA.
 
    <title>NSCountedSet class reference</title>
-   $Date: 2009-02-23 20:42:32 +0000 (Mon, 23 Feb 2009) $ $Revision: 27962 $
+   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
    */
 
-#import "config.h"
+#import "common.h"
 #import "GNUstepBase/GSLock.h"
 #import "Foundation/NSEnumerator.h"
 #import "Foundation/NSSet.h"
 #import "Foundation/NSCoder.h"
 #import "Foundation/NSArray.h"
-#import "Foundation/NSString.h"
 #import "Foundation/NSLock.h"
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSThread.h"
-#import "Foundation/NSObjCRuntime.h"
+#import "GNUstepBase/NSObject+GNUstepBase.h"
 
 @class	GSCountedSet;
 @interface GSCountedSet : NSObject	// Help the compiler
@@ -116,11 +115,11 @@ static Class NSCountedSet_concrete_class;
 - (id) initWithCoder: (NSCoder*)aCoder
 {
   unsigned	count;
-  Class		c = GSObjCClass(self);
+  Class		c = object_getClass(self);
 
   if (c == NSCountedSet_abstract_class)
     {
-      RELEASE(self);
+      DESTROY(self);
       self = [NSCountedSet_concrete_class allocWithZone: NSDefaultMallocZone()];
       return [self initWithCoder: aCoder];
     }
