@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistantObject class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-09-01 11:55:14 +0100 (Wed, 01 Sep 2010) $ $Revision: 31226 $
    */
 
 #import "common.h"
@@ -130,7 +130,7 @@ enum proxyLocation
 + (id) initWithCoder: (NSCoder*)aCoder;
 + (id) initWithLocal: (id)anObject connection: (NSConnection*)aConnection;
 + (id) initWithTarget: (unsigned)target connection: (NSConnection*)aConnection;
-+ (void) autorelease;
++ (id) autorelease;
 + (void) release;
 + (id) retain;
 + (BOOL) respondsToSelector: (SEL)sel;
@@ -138,8 +138,9 @@ enum proxyLocation
 
 @implementation	GSDistantObjectPlaceHolder
 
-+ (void) autorelease
++ (id) autorelease
 {
+  return self;
 }
 
 + (void) release
@@ -669,6 +670,10 @@ enum proxyLocation
  */
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
 {
+  if (0 == aSelector)
+    {
+      return nil;
+    }
   if (_object != nil)
     {
       return [_object methodSignatureForSelector: aSelector];
@@ -717,7 +722,7 @@ enum proxyLocation
 	  if ((uintptr_t)object_getClass(_protocol) == 0x2)
 	    {
 	      extern struct objc_method_description*
-		GSDescriptionForInstanceMethod();
+		GSDescriptionForInstanceMethod(Protocol *self, SEL aSel);
 	      mth = GSDescriptionForInstanceMethod(_protocol, aSelector);
 	    }
 	  else
@@ -729,7 +734,7 @@ enum proxyLocation
 	      if ((uintptr_t)object_getClass(_protocol) == 0x2)
 		{
 		  extern struct objc_method_description*
-		    GSDescriptionForClassMethod();
+		    GSDescriptionForClassMethod(Protocol *self, SEL aSel);
 		  mth = GSDescriptionForClassMethod(_protocol, aSelector);
 		}
 	      else
