@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObjCRuntime class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-08-21 09:03:32 +0100 (Sat, 21 Aug 2010) $ $Revision: 31191 $
    */
 
 #import "common.h"
@@ -74,7 +74,7 @@ NSStringFromSelector(SEL aSelector)
 }
 
 /**
- * Returns the selector whose name is supplied in the
+ * Returns (creating if necessary) the selector whose name is supplied in the
  * aSelectorName argument, or 0 if a nil string is supplied.
  */
 SEL
@@ -88,7 +88,7 @@ NSSelectorFromString(NSString *aSelectorName)
       [aSelectorName getCString: buf
 		      maxLength: len + 1
 		       encoding: NSASCIIStringEncoding];
-      return GSSelectorFromName (buf);
+      return sel_registerName (buf);
     }
   return (SEL)0;
 }
