@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2010-06-10 11:13:51 +0100 (Thu, 10 Jun 2010) $ $Revision: 30649 $
+   $Date: 2010-09-08 10:00:34 +0100 (Wed, 08 Sep 2010) $ $Revision: 31249 $
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -1461,7 +1461,7 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 /**
  * Returns the method signature describing how the receiver would handle
  * a message with aSelector.
- * <br />Raises NSInvalidArgumentException if given a null selector.
+ * <br />Returns nil if given a null selector.
  */
 - (NSMethodSignature*) methodSignatureForSelector: (SEL)aSelector
 {
@@ -1469,9 +1469,10 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
   struct objc_method	*mth;
   Class			c;
 
-  if (aSelector == 0)
-    [NSException raise: NSInvalidArgumentException
-		format: @"%@ null selector given", NSStringFromSelector(_cmd)];
+  if (0 == aSelector)
+    {
+      return nil;
+    }
 
   c = (GSObjCIsInstance(self) ? object_getClass(self) : (Class)self);
   mth = GSGetMethod(c, aSelector, GSObjCIsInstance(self), YES);
@@ -2312,110 +2313,29 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 		      YES));
 }
 
-+ (NSInteger) streamVersion: (TypedStream*)aStream
++ (NSInteger) streamVersion: (void*)aStream
 {
-#if !defined(NeXT_RUNTIME) && !defined(__GNUSTEP_RUNTIME__)
-  if (aStream->mode == OBJC_READONLY)
-    return objc_get_stream_class_version (aStream, self);
-  else
-#endif
-    return class_get_version (self);
+  GSOnceMLog(@"[NSObject+streamVersion:] is deprecated ... do not use");
+  return class_get_version (self);
 }
-
-//NOTE: original comments included the following excerpt, however it is
-//      probably not relevant now since the implementations are stubbed out.
-//  Subclasses should extend these, by calling
-//  [super read/write: aStream] before doing their own archiving.  These
-//  methods are private, in the sense that they should only be called from
-//  subclasses.
-
-/**
- * Originally used to read the instance variables declared in this
- * particular part of the object from a stream.  Currently stubbed out.
- */
-- (id) read: (TypedStream*)aStream
+- (id) read: (void*)aStream
 {
-  // [super read: aStream];
+  GSOnceMLog(@"[NSObject-read:] is deprecated ... do not use");
   return self;
 }
-
-/**
- * Originally used to write the instance variables declared in this
- * particular part of the object to a stream.  Currently stubbed out.
- */
-- (id) write: (TypedStream*)aStream
+- (id) write: (void*)aStream
 {
-  // [super write: aStream];
+  GSOnceMLog(@"[NSObject-write:] is deprecated ... do not use");
   return self;
 }
-
-/**
- * Originally used before [NSCoder] and related classes existed.  Currently
- * stubbed out.
- */
 - (id) awake
 {
-  // [super awake];
+  GSOnceMLog(@"[NSObject-awake] is deprecated ... do not use");
   return self;
 }
 
 @end
 
-/*
- * Stuff for compatibility with 'Object' derived classes.
- */
-@interface	Object (NSObjectCompat)
-+ (NSString*) description;
-+ (void) release;
-+ (id) retain;
-- (NSString*) className;
-- (NSString*) description;
-- (void) release;
-- (BOOL) respondsToSelector: (SEL)aSel;
-- (id) retain;
-@end
-
-@implementation	Object (NSObjectCompat)
-+ (NSString*) description
-{
-  return NSStringFromClass(self);
-}
-+ (void) release
-{
-  return;
-}
-+ (id) retain
-{
-  return self;
-}
-- (NSString*) className
-{
-  return NSStringFromClass([self class]);
-}
-- (NSString*) description
-{
-  return [NSString stringWithFormat: @"<%s: %p>",
-    GSClassNameFromObject(self), self];
-}
-- (BOOL) isProxy
-{
-  return NO;
-}
-- (void) release
-{
-  return;
-}
-- (BOOL) respondsToSelector: (SEL)aSelector
-{
-  /* Object implements -respondsTo: */
-  return [self respondsTo: aSelector];
-}
-- (id) retain
-{
-  return self;
-}
-@end
-
 
 
 @implementation	NSZombie
@@ -2450,6 +2370,10 @@ objc_create_block_classes_as_subclasses_of(Class super) __attribute__((weak));
 {
   Class	c;
 
+  if (0 == aSelector)
+    {
+      return nil;
+    }
   [allocationLock lock];
   c = NSMapGet(zombieMap, (void*)self);
   [allocationLock unlock];
