@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSProtocolChecker class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-08-21 09:03:32 +0100 (Sat, 21 Aug 2010) $ $Revision: 31191 $
    */
 
 #import "common.h"
@@ -74,8 +74,10 @@
 
 - (struct objc_method_description*) _methodDescription: (SEL)aSelector
 {
-  extern struct objc_method_description	*GSDescriptionForInstanceMethod();
-  extern struct objc_method_description	*GSDescriptionForClassMethod();
+  extern struct objc_method_description
+    *GSDescriptionForInstanceMethod(Protocol *self, SEL aSel);
+  extern struct objc_method_description
+    *GSDescriptionForClassMethod(Protocol *self, SEL aSel);
 
   if (_myProtocol != nil && _myTarget != nil)
     {
