@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSRunLoop class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-08-21 09:03:32 +0100 (Sat, 21 Aug 2010) $ $Revision: 31191 $
 */
 
 #import "common.h"
@@ -958,6 +958,8 @@ updateTimer(NSTimer *t, NSDate *d, NSTimeInterval now)
           unsigned              c;
           unsigned              i;
 
+	  ei = 0.0;	// Only needed to avoid compiler warning
+
 	  /*
 	   * Save current time so we don't keep redoing system call to
 	   * get it and so that we check timer fire dates against a known
@@ -1005,7 +1007,7 @@ updateTimer(NSTimer *t, NSDate *d, NSTimeInterval now)
 		    {
 		      GSIArrayRemoveItemAtIndexNoRelease(timers, i);
 		      [t fire];
-		      GSPrivateNotifyASAP(_currentMode); /* Post notifications. */
+		      GSPrivateNotifyASAP(_currentMode);
 		      IF_NO_GC([arp emptyPool]);
 		      if (updateTimer(t, d, now) == YES)
 			{
