@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSSet class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-07-03 07:08:28 +0100 (Sat, 03 Jul 2010) $ $Revision: 30914 $
    */
 
 #import "common.h"
@@ -946,7 +946,7 @@ static Class NSMutableSet_concrete_class;
 
 - (Class) classForCoder
 {
-  return NSMutableSet_concrete_class;
+  return NSMutableSet_abstract_class;
 }
 
 /** <init /> <override-subclass />
