@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTask class reference</title>
-   $Date: 2010-05-29 10:09:53 +0100 (Sat, 29 May 2010) $ $Revision: 30473 $
+   $Date: 2010-09-01 11:55:14 +0100 (Wed, 01 Sep 2010) $ $Revision: 31226 $
    */
 
 #import "common.h"
@@ -42,6 +42,7 @@
 #import "Foundation/NSNotification.h"
 #import "Foundation/NSNotificationQueue.h"
 #import "Foundation/NSTask.h"
+#import "Foundation/NSThread.h"
 #import "Foundation/NSTimer.h"
 #import "Foundation/NSLock.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
@@ -228,7 +229,6 @@ pty_slave(const char* name)
 
 @interface NSTask (Private)
 - (NSString *) _fullLaunchPath;
-- (void) _sendNotification;
 - (void) _collectChild;
 - (void) _terminatedChild: (int)status;
 @end
@@ -259,8 +259,30 @@ pty_slave(const char* name)
       if (tasksLock == nil)
         {
           tasksLock = [NSRecursiveLock new];
+	  /* The activeTasks map contains the NSTask objects corresponding
+	   * to running subtasks, and retains them until the subprocess
+	   * actually terminates.
+	   * The previous implementation stored non-retained objects and
+	   * the -finalize method removed them from the table, but this
+	   * caused a thread safety issue even though table access was
+	   * lock protected:
+	   * If thread t1 releases the task at the same time that the subtask
+	   * dies and is 'reaped' by t2, then there is a window such
+	   * that t1 can enter -dealloc while t2 is dealing with
+	   * subprocess termination notification.
+	   * So t1 completes deallocation while t2 performs
+	   * the notification, and then t2 tries to release the object...
+	   * t1: enter dealloc
+	   * t2: lookup task in activeTasks and retain/autorelease it
+	   * t2: create NSNotification (retaining task)
+	   * t2: post notification
+	   * t1: remove task from activeTasks
+	   * t1: release memory occupied by task object.
+	   * t2: complete notification ... attempt to release task object
+	   * but it's already deallocated.
+	   */
           activeTasks = NSCreateMapTable(NSIntMapKeyCallBacks,
-                NSNonOwnedPointerMapValueCallBacks, 0);
+                NSObjectMapValueCallBacks, 0);
         }
       [gnustep_global_lock unlock];
 
@@ -288,9 +310,7 @@ pty_slave(const char* name)
 
 - (void) finalize
 {
-  [tasksLock lock];
-  NSMapRemove(activeTasks, (void*)(intptr_t)_taskId);
-  [tasksLock unlock];
+  return;
 }
 
 - (void) dealloc
@@ -880,8 +900,20 @@ pty_slave(const char* name)
   return val;
 }
 
-- (void) _sendNotification
+- (void) _collectChild
 {
+  [self subclassResponsibility: _cmd];
+}
+
+- (void) _terminatedChild: (int)status
+{
+  [tasksLock lock];
+  IF_NO_GC([[self retain] autorelease];)
+  NSMapRemove(activeTasks, (void*)(intptr_t)_taskId);
+  [tasksLock unlock];
+  _terminationStatus = status;
+  _hasCollected = YES;
+  _hasTerminated = YES;
   if (_hasNotified == NO)
     {
       NSNotification	*n;
@@ -898,25 +930,6 @@ pty_slave(const char* name)
     }
 }
 
-- (void) _collectChild
-{
-  [self subclassResponsibility: _cmd];
-}
-
-- (void) _terminatedChild: (int)status
-{
-  [tasksLock lock];
-  NSMapRemove(activeTasks, (void*)(intptr_t)_taskId);
-  [tasksLock unlock];
-  _terminationStatus = status;
-  _hasCollected = YES;
-  _hasTerminated = YES;
-  if (_hasNotified == NO)
-    {
-      [self _sendNotification];
-    }
-}
-
 @end
 
 #ifdef __MINGW__
@@ -992,40 +1005,6 @@ GSPrivateCheckTasks()
   TerminateProcess(procInfo.hProcess, 10);
 }
 
-/*
- * Wait for child process completion.
- */
-static DWORD WINAPI _threadFunction(LPVOID t)
-{
-  DWORD	milliseconds = 60000;
-  int	taskId = [(NSTask*)t processIdentifier];
-
-  for (;;)
-    {
-      NSConcreteWindowsTask	*task;
-
-      [tasksLock lock];
-      task = (NSConcreteWindowsTask*)NSMapGet(activeTasks, (void*)(intptr_t) taskId);
-      [tasksLock unlock];
-      if (task == nil)
-	{
-	  return 0;	// Task gone away.
-	}
-      switch (WaitForSingleObject(task->procInfo.hProcess, milliseconds))
-	{
-	  case WAIT_OBJECT_0:
-	    handleSignal(0);	// Signal child process state change.
-	    return 0;
-
-	  case WAIT_TIMEOUT:
-	    break;		// Timeout ... retry
-
-	  default:
-	    return 0;		// Error ... stop watching.
-	}
-    }
-}
-
 
 static NSString*
 endSlashesDoubledFromString(NSString *aString)
@@ -1078,9 +1057,53 @@ quotedFromString(NSString *aString)
   return resultString;
 }
 
+
+/* Wait for child process completion.  The task object is retained by the
+ * current thread until it exits, but may be removed from the table of
+ * active tasks by another thread, so we must check that it's still
+ * active at intervals. 
+ */
+- (void) _windowsTaskWatcher
+{
+  void			*taskId = (void*)(intptr_t)[self processIdentifier];
+
+  for (;;)
+    {
+      NSConcreteWindowsTask	*task;
+
+      /* Check that this task is still active.
+       */
+      [tasksLock lock];
+      task = (NSConcreteWindowsTask*)NSMapGet(activeTasks, taskId);
+      [tasksLock unlock];
+      if (task != self)
+	{
+	  /* Task has been reaped by another thread ... so we can exit.
+	   */
+	  [NSThread exit];
+	}
+
+      /* Wait for up to 1 minute (60,000 milliseconds) to get a notification
+       * of an event from the subprocess.
+       */
+      if (WaitForSingleObject(procInfo.hProcess, (DWORD)60000)
+	!= WAIT_TIMEOUT)
+	{
+          /* The subprocess has terminated or failed in some way ...
+	   * Set flag so child task can be reaped, then exit since this
+	   * thread no longer needs to watch for events.
+	   */
+	  handleSignal(0);
+	  [NSThread exit];
+	}
+
+      /* Timed out ... repeat wait attempt.
+       */
+    }
+}
+
 - (void) launch
 {
-  DWORD			tid;
   STARTUPINFOW		start_info;
   NSString      	*lpath;
   NSString      	*arg;
@@ -1282,10 +1305,13 @@ quotedFromString(NSString *aString)
   [tasksLock lock];
   NSMapInsert(activeTasks, (void*)(intptr_t) _taskId, (void*)self);
   [tasksLock unlock];
+
   /*
    * Create thread to watch for termination of process.
    */
-  wThread = CreateThread(NULL, 0, _threadFunction, (LPVOID)self, 0, &tid);
+  [NSThread detachNewThreadSelector: @selector(_windowsTaskWatcher)
+			   toTarget: self
+			 withObject: nil];
 
   /*
    *	Close the ends of any pipes used by the child.
@@ -1344,6 +1370,7 @@ GSPrivateCheckTasks()
 #if	defined(WAITDEBUG)
 	      [tasksLock lock];
 	      t = (NSTask*)NSMapGet(activeTasks, (void*)(intptr_t)result);
+	      IF_NO_GC([[t retain] autorelease];)
 	      [tasksLock unlock];
 	      if (t != nil)
 		{
@@ -1356,7 +1383,7 @@ GSPrivateCheckTasks()
 	    {
 	      [tasksLock lock];
 	      t = (NSTask*)NSMapGet(activeTasks, (void*)(intptr_t)result);
-	      IF_NO_GC(AUTORELEASE(RETAIN(t));)
+	      IF_NO_GC([[t retain] autorelease];)
 	      [tasksLock unlock];
 	      if (t != nil)
 		{
