@@ -18,20 +18,18 @@
 
    */
 
-#include "config.h"
-#include	<string.h>
-#include	<GNUstepBase/preface.h>
-#include	<Foundation/NSObject.h>
-#include	<Foundation/NSArray.h>
-#include	<Foundation/NSDictionary.h>
-#include	<Foundation/NSEnumerator.h>
-#include	<Foundation/NSException.h>
-#include	<Foundation/NSString.h>
-#include	<Foundation/NSProcessInfo.h>
-#include	<Foundation/NSUserDefaults.h>
-#include	<Foundation/NSDebug.h>
-#include	<Foundation/NSAutoreleasePool.h>
-#include	<Foundation/NSPathUtilities.h>
+#include <string.h>
+
+#import "common.h"
+
+#import	"Foundation/NSArray.h"
+#import	"Foundation/NSDictionary.h"
+#import	"Foundation/NSEnumerator.h"
+#import	"Foundation/NSException.h"
+#import	"Foundation/NSProcessInfo.h"
+#import	"Foundation/NSUserDefaults.h"
+#import	"Foundation/NSAutoreleasePool.h"
+#import	"Foundation/NSPathUtilities.h"
 
 #define GSEXIT_SUCCESS EXIT_SUCCESS
 #define GSEXIT_FAILURE EXIT_FAILURE
@@ -135,7 +133,7 @@ main(int argc, char** argv, char **env)
   int derror = 0;
 
 #ifdef GS_PASS_ARGUMENTS
-  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
+  GSInitializeProcess(argc, argv, env);
 #endif
   [NSObject enableDoubleReleaseCheck: YES];
   pool = [NSAutoreleasePool new];
